function DashboardContexts(gtdinboxPage, elParent){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);
	this._eventHandlers = new GtdEventHandlers_Mutator(this);
	
	
	this.el = this._doc.createElement("DIV");
		this.el.className = "dashboard-contexts";
	this._eventObservers.observe( this.el, "mousedown", this._click, true );
	this.elMoreContexts = this._doc.createElement("DIV");
		this.elMoreContexts.className = "context-more";
		this.elMoreContexts.innerHTML = "<img src='chrome://gtdinbox/content/combo.png' />";
	this.elMoreContextsBox = this._doc.createElement("DIV");
		this.elMoreContextsBox.className = "context-more-box";	
	this._eventObservers.observe( this.elMoreContextsBox, "mousedown", this._click, true );
	this.popuper = new GtdPopuper( this._gtdinboxPage, this.elMoreContextsBox, this );
	
	this.refresh();
	
	if( elParent ) elParent.appendChild( this.el );
}

DashboardContexts.prototype.refresh = function(){
try{
	// Read in the contexts; arrange alphabetically and then prioritise by #clicks (stored in settings)
	// Show the first two (+ 'All') in the main view; and bury the rest in the MoreContextsBox
	
	var contextPrefix = this._gtdinboxPage.settings.labels.types.Contexts.prefix;
	
	this.el.innerHTML = "";
	this.elMoreContextsBox.innerHTML = "";
	var contexts = this._gtdinboxPage._labels.getLabelsArray("Contexts");
	if( !contexts || contexts.length==0 ) return;
	
	
	var elsByClicks = {};	// Index on #clicks and point to the elements with that many
	var clickList = [];
	var settingsContexts = this._gtdinboxPage.settings.contexts || {};
	
	// Create the All el:
	var el = this._doc.createElement("DIV");	
		el.innerHTML = this._gtdinboxPage.stringBundle.getString("Dashboard.Contexts.All");
		el.className = "context selected";
	this.el.appendChild( el );
	this.elSelected = el;
	
	// Create the els
	for( var i = 0; i < contexts.length; i++ ){
		var el = this._doc.createElement("DIV");
			el.className = "context";
			el.innerHTML = contexts[i].replace(contextPrefix, "");
			el.setAttribute("gtdLabelName", contexts[i]);
		var clicks = GIPROTOTYPE.padZero( (settingsContexts[contexts[i]]? (settingsContexts[contexts[i]].clicks || 0) : 0), 5);
		if( !elsByClicks[clicks] ){
			elsByClicks[clicks] = [];
			clickList.push(clicks);
		}
		elsByClicks[clicks].push(el);
	}
	
	// Now add them
	clickList.sort();
	var mainCount = 0;
	for( var i = 0; i < clickList.length; i++ ){
		var els = elsByClicks[clickList[i]];
		for( var j = 0; j < els.length; j++ ){
			if( mainCount++<2 ){
				this.el.appendChild( els[j] );	
			}else{
				this.elMoreContextsBox.appendChild( els[j] );
			}
		}
	}
	
	
	// Add the dropdown if needed:
	if(this.elMoreContextsBox.childNodes.length>0) this.el.appendChild( this.elMoreContexts );
	
	var elClear = this._doc.createElement("DIV");
		elClear.style.clear = "both";
	this.el.appendChild(elClear);
	
}catch(e){
	if( GTDDEBUGMODE ) alert("DashboardContexts refresh\n" + e.toString());
}
};

DashboardContexts.prototype._click = function(event){
	var el = GIPROTOTYPE.element(event);
	if( el.className=="context-more" ){
		this._clickMoreContexts(el);
	}else if( el.className=="context" ){
		this._clickContext(el);
	}
};

DashboardContexts.prototype._clickMoreContexts = function(el){
	// Reveal the box with more contexts as a popup that clicks off
	this.popup(this.elMoreContexts, {location:'bottomRight'});
};
DashboardContexts.prototype._clickContext = function(el){
	// Broadcast the selection
	// Update the UI
try{
	if( this.elSelected && this.elSelected.getAttribute("gtdLabelName")==el.getAttribute("gtdLabelName") ) return;	// No Change
	
	if( this.elSelected ) GIPROTOTYPE.removeClassName(this.elSelected, "selected");
	this.elSelected = el;
	GIPROTOTYPE.addClassName(this.elSelected, "selected");
	
	this._eventHandlers.broadcast("change-context");
	
	// Update click usage
	var context = this.context();
	if( context ){
		if( !this._gtdinboxPage.settings.contexts ) this._gtdinboxPage.settings.contexts = {};
		if( !this._gtdinboxPage.settings.contexts[context] ) this._gtdinboxPage.settings.contexts[context] = {clicks:0}
		this._gtdinboxPage.settings.contexts[context].clicks++;
		this._gtdinboxPage.saveSettings();
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("DashboardContexts clickContext\n" + e.toString());
}
};
DashboardContexts.prototype.context = function(){
	return this.elSelected? (this.elSelected.getAttribute("gtdLabelName") || "") : "";
};


DashboardContexts.prototype.destroy = function(){
	this._gtdinboxPage = null;
	this._doc = null;
	this._eventObservers.destroy();
	this._eventHandlers.destroy();
	this.popuper.destroy();
};