function FilesPopupUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(gtdinboxPage);
	
	this._create();
	
	this._popuper = new GtdPopuper(gtdinboxPage, this.el, this);
	this._popuper.addEventHandler(this, "popupOff", this._onPopupOff);
};

FilesPopupUI.prototype._create = function(){
	this.el = this._doc.createElement("DIV");
		this.el.className = "files-popup";
		this.el.style.display = "none";
	
	this.elResults = this._doc.createElement("TABLE");
		this.elResults.className = "result-container";
	this.el.appendChild( this.elResults );
	
	this.elPreview = this._doc.createElement("DIV");
		this.elPreview.className = "preview";
	this.el.appendChild( this.elPreview );

	this._eventObservers.observe( this.elResults, "mouseover", this._mouseoverResults, false );
	
	this._doc.body.appendChild( this.el );
};

FilesPopupUI.prototype.render = function(files, elPopupFrom){

try{

	
	var html = [];
	this._lineItems = {};
	this.elPopupFrom = elPopupFrom;
	this.elResults.innerHTML = "";
	
	for( var i = 0; i < files.length; i++ ){
		var id = (new Date()).valueOf() + files[i].name;
		this._lineItems[id] = files[i];
		
		var el = this._doc.createElement("TR");
			el.className = "line-item";
			el.setAttribute("gtdLineItem", id);
		if( (this.elResults.childNodes.length%2)!=0 ) el.className += " alternate";
			
		html.length = 0;
		html.push( "<td><img class='icon' src='" + files[i].icon + "' /></td>" );
		html.push( "<td>" + files[i].name + "</td>" );
		var daysAgo = parseInt(((new Date()).valueOf() - files[i].epoch) / (1000*60*60*24));
		html.push( "<td>" + daysAgo + " " + this._gtdinboxPage.stringBundle.getString("FilesUI.LineItem.DaysAgo") + "</td>" );
		html.push( "<td>" + files[i].body.sender[0] + "</td>" );
		html.push( "<td><span class='command' name='view-thread'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.LineItem.ViewThread") + "</span></td>" );
		html.push( "<td><span class='command' name='download'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.LineItem.Download") + "</span></td>" );
		this.elResults.appendChild( el );
		el.innerHTML = html.join("");
		
		if( i==0 ) this._setPreview(el);
	}
	
	this._popuper.resetDeadZones();
	this._popuper.addPopupChild( elPopupFrom );
	this.popup(elPopupFrom, {popOffMouseout:true});
	
	
}catch(e){
	if( GTDDEBUGMODE ) alert( "FilesPopupUI render\n" + e.toString() );
}
};

FilesPopupUI.prototype._onPopupOff = function(){
	this.elPopupFrom = null;
};

FilesPopupUI.prototype._mouseoverResults = function(event){

try{
	var el = GIPROTOTYPE.element(event);
	// Find the tr
	while( el.parentNode && el.nodeName!="TR" ){
		el = el.parentNode;
	}
	if( el.nodeName!="TR" ) return;
	
	this._setPreview(el);
}catch(e){
	if( GTDDEBUGMODE ) alert( "FilesPopupUI _mouseoverResults\n" + e.toString() );
}
};

FilesPopupUI.prototype._setPreview = function(elTR){
try{

	if( this._lastElTR==elTR ) return;
	
	var id = elTR.getAttribute("gtdLineItem");
	if( !id ) return;
	
	var file = this._lineItems[id];
	if( !file ) return;
	

	// Wipe out any other selected items
	for( var i = 0; i < this.elResults.childNodes.length; i++ ){
		if( GIPROTOTYPE.hasClassName(this.elResults.childNodes[i], "selected") ) GIPROTOTYPE.removeClassName( this.elResults.childNodes[i], "selected" );
	}
	this._lastElTR = elTR;
	this.elPreview.innerHTML = "";
	
	var previewing = false;	
	if( file.type.indexOf("image/")==0 ){
		this.elPreview.innerHTML = "<img src='" + file.preview + "' />"; // this._gtdinboxPage.gmailDOM.local("domain") + 
		previewing = true;
	}
	if( previewing ) GIPROTOTYPE.addClassName( elTR, "selected" );
	
	
}catch(e){
	if( GTDDEBUGMODE ) alert( "FilesPopupUI _setPreview\n" + e.toString() );
}
};

FilesPopupUI.prototype.destroy = function(){
	if( !this._gtdinboxPage ) return;
	
	this.el.parentNode.removeChild( this.el );	
	this._gtdinboxPage = null;
	this._popuper.destroy();
	this._eventObservers.destroy();
};