function FilesUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);
	this._files = null;
	
	this._create();
	
	this._filesPopup = new FilesPopupUI(gtdinboxPage);
}

FilesUI.prototype._create = function(){
	this.el = this._doc.createElement("DIV");
		this.el.className = "files";
	
	var elRankings = this._doc.createElement("DIV");
		elRankings.className = "rankings";
		elRankings.innerHTML += "<span class='ranking selected' name='recent'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.Rankings.Recent") + "</span>";
		elRankings.innerHTML += "<span class='ranking' name='contacts'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.Rankings.Contacts") + "</span>";
		elRankings.innerHTML += "<span class='ranking' name='threads'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.Rankings.Threads") + "</span>";
	this.el.appendChild( elRankings );
	
	this.elResults = this._doc.createElement("TABLE");
		this.elResults.className = "result-container";
	this.el.appendChild(this.elResults);
	
	this._eventObservers.observe( this.elResults, "mouseover", this._mouseoverResults, false);
	this._eventObservers.observe( this.elResults, "click", this._clickResults, false);
	this._eventObservers.observe( elRankings, "click", this._clickRankings, false);
	
	return this.el;
};

FilesUI.prototype.load = function(search){
	// Load the threadList from the search and pass to loadThreadList
try{
	
	this._loading(true);
	if( !this._searcher ) this._searcher = new Searcher(this._gtdinboxPage);
	this._searcher.search( search, GIPROTOTYPE.bind(this, this.loadThreadList) );
}catch(e){
	if(GTDDEBUGMODE) alert("FilesUI load\n" + e.toString());
}
};

FilesUI.prototype.loadThreadList = function(threadList){
	// Process each thread, building up a list of files {name} = [file, file]
try{	
	this._loading(true);
	threadList.getFiles(null, GIPROTOTYPE.bind(this, function(files){
			this._files = files;
			this._renderRankedFiles();
		}));
}catch(e){
	if(GTDDEBUGMODE) alert("FilesUI loadThreadList\n" + e.toString());
}
};

FilesUI.prototype._renderRankedFiles = function(rank){
	// Clear out the result list and render
	// 'recent': a simple list
	// 'contact': divide into contacts
	// 'threads': divide into threads

try{	
	if( !rank ) rank = this._lastRank || "recent";
	this._lastRank = rank;
	this._loading(false);
	this.elResults.innerHTML = "";
	
	this._lineItemHash = {};
	
	switch(rank){
		case "recent":
			for( var i = 0; i < this._files.length; i++ ){
				this._addFile( this._files[i][0], this._files[i] );
			}
			break;
		case "contacts":
			// Break into a hash of contacts
			var contactsHash = {};
			for( var i = 0; i < this._files.length; i++ ){
				for( j = 0; j < this._files[i].length; j++ ){
					var file = this._files[i][j];
					var name = file.body.sender;
					if( !contactsHash[name[1]] ) contactsHash[name[1]] = {name:name[0], email:name[1], files:{}};
					if( !contactsHash[name[1]].files[file.name] ) contactsHash[name[1]].files[file.name] = [];
					contactsHash[name[1]].files[file.name].push( file );
				}
			}
			for( c in contactsHash ){
				var contact = contactsHash[c];
				this._addTitle( contact.name + " (" + contact.email + ")" );
				for( name in contact.files){
					this._addFile( contact.files[name][0], contact.files[name] );
				}
			}
			break;
		case "threads":
			var threadsHash = {};
			for( var i = 0; i < this._files.length; i++ ){
				for( j = 0; j < this._files[i].length; j++ ){
					var file = this._files[i][j];
					var subject = file.thread.subject;
					if( !threadsHash[subject] ) threadsHash[subject] = {'subject':subject, files:{}};
					if( !threadsHash[subject].files[file.name] ) threadsHash[subject].files[file.name] = [];
					threadsHash[subject].files[file.name].push( file );
				}
			}
			for( c in threadsHash ){
				var threadData = threadsHash[c];
				this._addTitle( threadData.subject );
				for( name in threadData.files){
					this._addFile( threadData.files[name][0], threadData.files[name] );
				}
			}
			break;
	}
}catch(e){
	if(GTDDEBUGMODE) alert("FilesUI _renderRankedFiles\n" + e.toString());
}
};

FilesUI.prototype._loading = function(v){
	if( this._elLoading ){
		this._elLoading.parentNode.removeChild( this._elLoading );
		this._elLoading = null;
	}
		
	if( v ){
		this._elLoading = this._doc.createElement("IMG");
			this._elLoading.className = "loading";
			this._elLoading.src = "chrome://gtdinbox/skin/indicator_medium.gif"
		this.el.appendChild( this._elLoading );
	}
};

FilesUI.prototype._addTitle = function(title){
	var el = this._doc.createElement("TR");
		el.className = "title";
	this.elResults.appendChild( el );
	el.innerHTML = "<td colspan='6'>" + title + "</td>";
};

FilesUI.prototype._addFile = function(file, files){
try{
	var children = files? files.length : 0;
	
	var id = (new Date()).valueOf() + file.name;
	this._lineItemHash[id] = files;

	var el = this._doc.createElement("TR");
		el.className = "line-item";
		el.setAttribute("gtdLineItem", id);
	if( (this.elResults.childNodes.length%2)!=0 ) el.className += " alternate";
		
	var html = [];
	html.push( "<td><img class='icon' src='" + file.icon + "' /></td>" );
	html.push( "<td>" + file.name + "</td>" );
	var daysAgo = parseInt(((new Date()).valueOf() - file.epoch) / (1000*60*60*24));
	html.push( "<td>" + daysAgo + " " + this._gtdinboxPage.stringBundle.getString("FilesUI.LineItem.DaysAgo") + "</td>" );
	html.push( "<td>" + (children? children : "") + "</td>" );
	html.push( "<td><span class='command' name='view-thread'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.LineItem.ViewThread") + "</span></td>" );
	html.push( "<td><span class='command' name='download'>" + this._gtdinboxPage.stringBundle.getString("FilesUI.LineItem.Download") + "</span></td>" );
	this.elResults.appendChild( el );
	el.innerHTML = html.join("");
	
}catch(e){
	if(GTDDEBUGMODE) alert("FilesUI _addFile\n" + e.toString());
}
};

FilesUI.prototype._clickRankings = function(event){
	// Extract the element
	// Refresh the rank
	// Update the selected item
	
	var el = GIPROTOTYPE.element(event);
	if( !GIPROTOTYPE.hasClassName( el, "ranking" ) ) return;
	var elRankings = el.parentNode;
	
	// Remove other selected
	for( var i = 0; i < elRankings.childNodes.length; i++ ){
		GIPROTOTYPE.removeClassName( elRankings.childNodes[i], "selected" );
	}
	GIPROTOTYPE.addClassName( el, "selected" );
	
	this._renderRankedFiles( el.getAttribute("name") );
};

FilesUI.prototype._clickResults = function(event){
try{
	var el = GIPROTOTYPE.element(event);
	if( !GIPROTOTYPE.hasClassName(el, "command") ) return;
	
	var elTR = el.parentNode;
	while( elTR.parentNode && elTR.nodeName!="TR" ){
		elTR = elTR.parentNode;
	}
	if( elTR.nodeName!="TR" ) return;
	
	var files = this._lineItemHash[ elTR.getAttribute("gtdLineItem") ];
		
	switch( el.getAttribute("name") ){
		case "view-thread":
			var url = this._gtdinboxPage.gmailDOM.local("domain") + "#inbox/" + files[0].thread.id;
			if( event.button==0 ){
				this._doc.defaultView.top.location.href = url;
			}else if(event.button==1){
				gBrowser.selectedTab = gBrowser.addTab(url);
			}
			break;
		case "download":
			var url = this._gtdinboxPage.gmailDOM.local("domain") + files[0].url;
			gBrowser.selectedTab = gBrowser.addTab(files[0].url);
			break;
	}
}catch(e){
	if( GTDDEBUGMODE ) alert( "FilesUI _clickResults\n" + e.toString() );
}
};

FilesUI.prototype._mouseoverResults = function(event){
try{
	var el = GIPROTOTYPE.element(event);
	// Find the tr
	while( el.parentNode && el.nodeName!="TR" ){
		el = el.parentNode;
	}
	if( el.nodeName!="TR" ) return;
	
	if( this._filesPopup.elPopupFrom==el ) return;
	
	var id = el.getAttribute("gtdLineItem");
	if( !id ) return;
	
	var files = this._lineItemHash[ id ];
	this._filesPopup.render(files, el);

}catch(e){
	if( GTDDEBUGMODE ) alert( "FilesUI _mouseoverResults\n" + e.toString() );
}
};

FilesUI.prototype.destroy = function(){
	if( !this._gtdinboxPage ) return;
	
	this.el.parentNode.removeChild( this.el );
	this._gtdinboxPage = null;
	this._searcher.destroy();
	this._eventObservers.destroy();
};