function PrioritiseByContacts(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(gtdinboxPage);
	this._searcher = new Searcher(gtdinboxPage);
	this._closeContacts = new CloseContacts(gtdinboxPage);
	this._buckets = {};
	
	this._create();	
};

PrioritiseByContacts.prototype._create = function(){
	this.el = this._doc.createElement("DIV");
		this.el.className = "prioritise-by-contacts";
		this.el.style.className = "none";
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.className = "title";
		elTitle.innerHTML = "<span class='title-text'>" + this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.Title") + "</span>";
	
	var elClose = this._doc.createElement("IMG");
		elClose.src = "chrome://gtdinbox/content/cross.png";
		elClose.className = "close-button";
		elClose.style.position = "absolute";
		elClose.style.right = "0px";
		elClose.style.top = "0px";
		this._eventObservers.observe(elClose, "mousedown", this._clickClose, false);
	elTitle.appendChild(elClose);
	this.el.appendChild( elTitle );
	
	var elControl = this._doc.createElement("DIV");
		elControl.className = "control-panel";
	var elRankings = this._doc.createElement("DIV");
		elRankings.className = "rankings";
		elRankings.innerHTML += "<span class='ranking' name='closeness'>" + this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.Rankings.Closeness") + "</span>, ";
		elRankings.innerHTML += "<span class='ranking' name='threads'>" + this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.Rankings.Threads") + "</span>, ";
	elControl.appendChild( elRankings );
	
	var elRefine = this._doc.createElement("DIV");
		elRefine.className = "refine";
		elRefine.innerHTML += "<div classname='checkbox'><input type='checkbox' name='unread' checked='1' /> " + this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.Refine.UnreadOnly") + "</div>";
		//elRefine.innerHTML += "<div classname='checkbox'><input type='checkbox' name='recent-projects' /> " + this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.Refine.RecentProjects") + "</div>";
		elRefine.innerHTML += "<button name='refresh'>" + this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.Refine.Refresh") + "</button>";
	elControl.appendChild( elRefine );
	this.el.appendChild( elControl );
	
	this.elResults = this._doc.createElement("DIV");
		this.elResults.className = "result-container";
	this.el.appendChild( this.elResults );
	
	this._eventObservers.observe(elRankings, "mousedown", this._clickRankings, false);
	this._eventObservers.observe(elRefine, "mousedown", this._clickRefine, false);
	
};

PrioritiseByContacts.prototype._getCheckboxes = function(){
	var elCheckbox = {};
	var elCheckboxes = this.el.getElementsByTagName("input");
	for( var i = 0; i < elCheckboxes.length; i++ ){
		var el = elCheckboxes[i];
		if( el.getAttribute("type")=="checkbox" ){
			elCheckbox[ el.getAttribute("name") ] = el;
		}
	}
	return elCheckbox;
};

PrioritiseByContacts.prototype.refresh = function(){
	// Search for all (unread) inbox messages in the last 3 months
	//		(If recent projects only; short list to recent projects as stored)
	// Load data into contact buckets
	// For each contact, load the closeness; and merge the closeness metric into the main contact buckets
	// Rank accordingly
	
	
	var elCheckboxes = this._getCheckboxes();
	
	var d = (new Date()).valueOf() - (1000*60*60*24*90);
	d = new Date(d);	
	var query = "in:inbox after:" + d.getFullYear() + "/" + (d.getMonth() + 1) + "/" + d.getDate()
	if( elCheckboxes.unread.checked ) query += " is:unread";
	/*
	if( elCheckbox["recent-projects"].checked ){	// Load contacts that worked on recent projects so we can restrict to them
		var projects = this._gtdinboxPage._recentItems.projects();
		if( projects.length > 0 ){
			query += " (";
			for( var i = 0; i < projects.length; i++ ){
				if( i > 0 ) query += " OR ";
				query += GIPROTOTYPE.searchableLabel(projects[i]);
			}
			query += ")";
		}
	}
	*/

	this.elResults.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif' class='loading'/>";
	
	this._searcher.search(query, GIPROTOTYPE.bind(this, function(threadList){
			this._searcher.renderThreadListByType("Contact", threadList, null, null, null, GIPROTOTYPE.bind(this, function(type, count, buckets){
					// Find the unique contacts in the buckets and assess closeness
					var contacts = [];
					for( b in buckets ) contacts.push(b);
					this._closeContacts.processContacts(null, contacts, GIPROTOTYPE.bind(this, function(contacts, contactsHash){
							// Add the closeness data in the new buckets to the old buckets
							for( b in contactsHash ){
								buckets[b].closeness = contactsHash[b].seen;
								buckets[b].closenessImgSrc = contactsHash[b].closenessImgSrc;
								buckets[b].closenessTooltip = contactsHash[b].closenessTooltip;
							}
							this._buckets = buckets;
							this._renderByRank("closeness");
						}));
				}));
			
		}));	
};

PrioritiseByContacts.prototype._renderByRank = function(rank){
	// Clear out elResults
	// For each contact in buckets, sorted by 'rank', add the item
	
	this.elResults.innerHTML = "";
	
	d = new Date(d);	
	var parentSearch = "in:inbox after:" + d.getFullYear() + "/" + (d.getMonth() + 1) + "/" + d.getDate()
	var elCheckboxes = this._getCheckboxes();
	if( elCheckboxes.unread.checked ) parentSearch += "is:unread";
	
	var rank = [];
	var rankHash = {};
	for( b in this._buckets ){
		var id = null;
		switch( rank ){
			case "closeness":
				id = GIPROTOTYPE.padZero( this._buckets[b].closeness, 5 ) + this._buckets[b].name;
				break;
			case "threads":
				id = GIPROTOTYPE.padZero( this._buckets[b].seen, 5 ) + this._buckets[b].name;
				break;
		}
		rank.push(id);
		rankHash[id] = this._buckets[b];
	}
	rank.sort().reverse();
		
	for( var i = 0; i < rank.length; i++ ){
		var bucket = rankHash[rank[i]];
		this._addContact( bucket, rank, parentSearch );
	}
};

PrioritiseByContacts.prototype._addContact = function(bucket, rank, parentSearch){
	// The bucket is the contact details
	// Create an el, with preview-thread and popout commands
	
	var el = this._doc.createElement("DIV");
		el.className = "line-item";
	this.elResults.appendChild( el );
	
	var elBody = this._doc.createElement("DIV");
		elBody.className = "body";
		var count = bucket.seen;
		if( rank=="closeness) count = bucket.closeness;
		elBody.innerHTML = bucket.displayName + " (" + count + ")";
	el.appendChild( elBody );
	
	var fAttribs = function(el){
		el.setAttribute("gtdSearch", search);
		if( name ) el.setAttribute("gtdName", bucket.name);
		if( type ) el.setAttribute("gtdType", "Contact");
	};
	fAttribs(el);
	fAttribs(elBody);
	
	var elCmds = this._doc.createElement("DIV");
		elCmds.className = "commands";
	el.appendChild( elCmds );
		
	// Popout
	var elExpander = this._doc.createElement("IMG");
		elExpander.className = "expander";
		elExpander.src = "chrome://gtdinbox/content/filter-extend.png";
		elExpander.title = this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.LineItem.Cmds.Expand.ToolTip");	
	elCmds.appendChild( elExpander );
	fAttribs(elExpander);
	
	// Show Threads
	var elShowThreads = this._doc.createElement("IMG");
		elShowThreads.className = "show-threads";
		elShowThreads.src = "chrome://gtdinbox/content/show-threads.png";
		elShowThreads.title = this._gtdinboxPage.stringBundle.getString("PrioritiseByContacts.LineItem.Cmds.ShowThreads.ToolTip");	
	elCmds.appendChild( elShowThreads );
	fAttribs(elShowThreads);
	
	var elPreview = this._doc.createElement("DIV");
		elPreview.className = "show-threads-preview-box";
		elPreview.style.display = "none";
	elParent.appendChild( elPreview );
	
	var elClear = this._doc.createElement("DIV");
		elClear.style.clear = "both";
		elClear.style.height = "1px";
	elLink.appendChild( elClear );
	
	return el;
};

PrioritiseByContacts.prototype._clickRankings = function(event){
	var el = GIPROTOTYPE.element(event);
	if( !el.getAttribute("name") ) return;
	switch( el.getAttribute("name") ){
		case "closeness":
			break;
		case "threads":
			break;
	}
};

PrioritiseByContacts.prototype._clickRefine = function(event){
	var el = GIPROTOTYPE.element(event);
	if( !el.getAttribute("name") ) return;
	switch( el.getAttribute("name") ){
		case "unread":
			break;
		case "recent-projects":
			break;
	}
};

PrioritiseByContacts.prototype._clickClose = function(event){
	this.show(false);
};

PrioritiseByContacts.prototype.show = function(v){
	if( typeof v =="undefined" ) return !(this.el.style.display=="none");
	this.el.style.display = v? "":"none";
};

PrioritiseByContacts.prototype.destroy = function(){
	if( !this._gtdinboxPage ) return;
	
	this._gtdinboxPage = null;
	this._doc = null;
	this._eventObservers.destroy();
	this._searcher.destroy();
	this._closeContacts.destroy();
};