/* Copyright Andy Mitchell 2007. No warranties */

function ThreadList(gtdinboxPage){
	this.gtdinboxPage = gtdinboxPage;
	this._threads = [];
	this._threadsHashId = {};
	this._threadsHashRow = {};
	this._noSubject = "gtdnosubject";
	this._noSubjectRegExp = /^(\s*\([^)]+\)\s*)|(\w+\:)$/;
	this._multiToOne = new MultiToOne(this.gtdinboxPage, 5000);
}

ThreadList.prototype.getThreadById = function(id, fCallback, forceUpdate){
	
	if( !id ){
		if( fCallback ) return fCallback()
		else return null;
	}
	if( fCallback ){	// Look it up more deeply
		var thread = this._threadsHashId[id];
		if( thread && !forceUpdate && thread.messageCount > 0 && thread.messageCount<=thread.messages().length ){	// We have it whole
			fCallback(thread);
		}else{	// Need to retrieve it
			this.gtdinboxPage.gmailDOM.getThreadData([id], this, GIPROTOTYPE.bind(this, function(){
					fCallback(this._threadsHashId[id]);
				}));
		}
	}else{
		if( this._threadsHashId[id] ) return this._threadsHashId[id];
	}
};


ThreadList.prototype.getThreadByRow = function(subject, snippetFromHTML, namesArray, fCallback, retrying, threadBefore, threadAfter){
	// Retrieve the subject first. If multiple ones found; discern first on snippet; and then on author. 
	// TODO Improvement: If nothing found, extract the range from the UI (1-50), the search from the URL (inbox/sent items/search), pull in all the relevant threads; and then deep-pull the desired thread.
	
	
try{
	//this.gtdinboxPage.unsafeWindow.console.log("getThreadByRow: " + subject);
	//this.gtdinboxPage.unsafeWindow.console.log(this._threads);
	//this.gtdinboxPage.unsafeWindow.console.log(this._threadsHashRow);
	
	if( !retrying ) retrying = 0;
	
	var threads = this._threadsHashRow[subject];
	if( !threads && this._noSubjectRegExp.test(subject) ) threads = this._threadsHashRow[this._noSubject];	// 'subject' could be (no subject)
		
	if( !threads || threads.length==0 ){	// Nothing found, check the subject is not so long as to be indiscoverable
		if( subject.substring(subject.lastIndexOf("..."))=="..." ){	// has abbreviated ending; means subject was cut short
			subject = subject.substring(0, subject.lastIndexOf("...")); 	// strip trailing abbreviation, ...
			threads = [];
			for( s in this._threadsHashRow ){
				if( s.indexOf(subject)==0 ){
					threads = threads.concat( this._threadsHashRow[s] );
				}
			}
		}
	}
	
	//this.gtdinboxPage.unsafeWindow.console.log("Tested Subject:");
	//this.gtdinboxPage.unsafeWindow.console.log(threads);
	
	if( !threads ){
		if( fCallback && retrying<=1 ) return this.getThreadByRowFail(subject, snippetFromHTML, namesArray, fCallback, retrying);
		else return;
	}else{
		if( threads.length==1 ){
			if( fCallback ) return this.getThreadById( threads[0].id, fCallback );
			else return threads[0];
		}
	}
	
	// Try to distinguish on author
	if( !(namesArray instanceof Array) ) namesArray = [namesArray];
	var threads2 = [];
	for( var i = 0; i < threads.length; i++ ){
		if( threads[i].testNames(namesArray) ) threads2.push(threads[i]);
	}

	//this.gtdinboxPage.unsafeWindow.console.log("Tested Names:");
	//this.gtdinboxPage.unsafeWindow.console.log(namesArray);
	//this.gtdinboxPage.unsafeWindow.console.log(threads2);

	if( threads2.length==0 ){
		if( fCallback && retrying<=1 ) return this.getThreadByRowFail(subject, snippetFromHTML, namesArray, fCallback, retrying);
		else return;
	}else{
		if( threads2.length==1 ){
			if( fCallback ) return this.getThreadById( threads2[0].id, fCallback );
			else return threads2[0];
		}
	}
	
	// Now try to decipher. HTML snippet can be shorter than real snippet. So, tokenise words in HTML snippet and see if they're found in other snippet. 
	// TODO Fails on search; because it doesn't reveal the last message
	var threads3 = [];
	for( var i = 0; i < threads2.length; i++ ){
		if( threads2[i].testSnippet(snippetFromHTML) ) threads3.push(threads2[i]);
	}
	
	//this.gtdinboxPage.unsafeWindow.console.log("Tested Snippet:");
	//this.gtdinboxPage.unsafeWindow.console.log(threads3);	
	
	if( threads3.length==1 ){
		if( fCallback ) return this.getThreadById( threads3[0].id, fCallback );
		else return threads3[0];
	}
	/*
	if( threads3.length==0 && fCallback && !retrying ){
		return this.getThreadByRowIndex( subject, snippetFromHTML, namesArray, fCallback );
	}else if( threads3.length > 1 && (threadBefore || threadAfter) ){	// Tried to load more, failed. Try those around it. 
		for( var i = 0; i < threads3.length; i++ ){
			if( (!threadBefore || (threads3[i].epoch() < threadBefore.epoch())) && (!threadAfter || (threads3[i].epoch() > threadAfter.epoch())) ){
				if( fCallback ) return this.getThreadById( threads3[i].id, fCallback );
				else return threads3[i];
			}
		}
	}
	*/
	if( fCallback ){
		//fCallback();
		this.getThreadByRowIndex(fCallback);
	}else{
		return;
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadList getThreadByRow\n" + e.toString());
}
	
};
ThreadList.prototype.getThreadByRowFail = function(subject, snippetFromHTML, namesArray, fCallback, retrying){
	// Extract the range from the UI, the search from the URL, and repeat the thread search; then grab the desired thread. 
try{
	if( !retrying ) retrying = 0;
	
	var range = this.gtdinboxPage.gmailDOM.parseElement( "threadlist-range", this.gtdinboxPage.gmailDOM.element("threadlist-range") );
	var searchStr = this.gtdinboxPage.gmailDOM.extractSearchQuery();
		
	this.gtdinboxPage.gmailDOM.search(searchStr, this, range.start, range.end - range.start, GIPROTOTYPE.bind(this, function(searchArray, query, threadList, currentOffset){
			this.parseAjaxResponse(searchArray[0]);threadList.parseAjaxResponse(searchArray[0]);
	
			this.getThreadByRow(subject, snippetFromHTML, namesArray, fCallback, ++retrying);
		}));
		
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadList getThreadByRowFail\n" + e.toString());
}
	
};

ThreadList.prototype.getThreadByRowIndex = function(fCallback){
	// Get the first thread that is checked
	
	
	var range = this.gtdinboxPage.gmailDOM.parseElement( "threadlist-range", this.gtdinboxPage.gmailDOM.element("threadlist-range") );
	var searchStr = this.gtdinboxPage.gmailDOM.extractSearchQuery();
	var threadList = new ThreadList(this.gtdinboxPage);
	
	this.gtdinboxPage.gmailDOM.search(searchStr, threadList, range.start, range.end - range.start, GIPROTOTYPE.bind(this, function(searchArray, query, threadList, currentOffset){
			threadList.parseAjaxResponse(searchArray[0]);
			var rowIndex = this.gtdinboxPage.gmailDOM.parseElement("thread-list-index")[0];
						
			fCallback( threadList._threads[rowIndex] );
		}));
	
};

ThreadList.prototype.parseAjaxResponse = function(a){
	var thread = null;
	var found = false;
	
	
	var c = this._threads.length;
	for( var i = 0; i < a.length; i++ ){
		if( a[i][0]==="tb" ){	// it's a set of 10 rows; relating to UI row data
			for( var j = 3; j < a[i].length; j++ ){
				var titem = a[i][j];
				if( thread = this._threadsHashId[titem[0]] ){	// exists
					thread.update("tb", titem);
				}else{	// create
					thread = new Thread(this, titem[0], "tb", titem);
					this.addThread(thread);
				}
				this.addToSubjectHash(thread);
			}
			found = true;
		}else if( a[i][0]==="cs" ){	// start of a block of message data items (cs is conversation)
			var titem = a[i];
			if( this._threadsHashId[titem[1]] ){
				thread = this._threadsHashId[titem[1]];
			}else{
				thread = new Thread(this, titem[1]);
				this.addThread(thread);
			}
			thread.messageCount = a[i][3];
			found = true;
		}else if( a[i][0]==="ms" ){
			var titem = a[i];
			if( thread ){
				thread.update("ms", titem);
				if( !a[i+1] || a[i+1][0]!="ms" ){	// this is last msg in thread; so subject/snippet is correct
					//thread.checkAutoLabel();
				}
			}
			this.addToSubjectHash(thread);
			found = true;
		}else{
			thread = null;
		}
	}
	if( found ){
		this._multiToOne.process("update-threads", GIPROTOTYPE.bind(this, this.syncDbAll) );
	}
	
	return this._threads.length - c;	// return a count
};

ThreadList.prototype.syncDbAll = function(){
	// We currently store labels against contacts (and threads); but only when we label/unlabel.
	// If a new user replies to a thread, or we forward it, they should also be associated with the label.
	// So, when a thread updates, we re-sync names with db.
try{
	var j = 0;
	for( var i = 0; i < this._threads.length; i++ ){
		var t = this._threads[i];
		if( t.labels.length > 0 && !t.syncedDbSinceUpdate ){
			t.syncDb();
			j++;
		}
	};
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadList syncDbAll\n" + e.toString());
}
};


ThreadList.prototype.addThread = function(thread, incSubject){
	this._threads.push(thread);
	this._threadsHashId[thread.id] = thread;
	
};
ThreadList.prototype.addToSubjectHash = function(thread){
	var subject = thread.subject;
	if( !subject || this._noSubjectRegExp.test(subject) ) subject = this._noSubject;
	if( !subject ) return;
	
	var a = null;
	if( a = this._threadsHashRow[subject] ){
		for( var i = 0; i < a.length; i++ ) if(a[i]==thread) return;
		this._threadsHashRow[subject].push(thread);
	}else{
		this._threadsHashRow[subject] = [thread];
	}
};

ThreadList.prototype.hasFullBodiedThreads = function(){
	// Detect if any thread with previews exist - a way to tell if threads are being programmatically read
	
	var found = false;
	for( var i = 0; i < this._threads.length; i++ ){
		if( this._threads[i].getLatestMsg() ){
			found = true;
			break;
		}
	}
	
	return found;
};

ThreadList.prototype.getFiles = function(threads, fCallback, isRetry){
	// Organise the files by date, then aggregate any shared names
try{
	if( !threads ) threads = this._threads;
	if( threads.length==0 ){
		if( fCallback ){
			return fCallback([]);
		}else{
			return [];
		}
	}
		
	if( threads[0].files().length==0 && fCallback && !isRetry ){	// Test these threads are loaded with files
		// Load each thread in depth and callback
		var threadIds = [];
		for( var i = 0; i < threads.length; i++ ) threadIds.push( threads[i].id );
		this.gtdinboxPage.gmailDOM.getThreadData(threadIds, this, GIPROTOTYPE.bind(this, function(){
				this.getFiles(threads, fCallback, true);
			}));
		return;
	}else if( threads[0].files().length==0 && isRetry ){
		if( fCallback ) return fCallback([]);
		else return [];
	}
	
	var filesHash = {};
	for( var i = 0; i < threads.length; i++ ){
		var files = threads[i].files();
		for( var j = 0; j < files.length; j++ ){
			var name = files[j].name;
			if( !filesHash[name] ){
				filesHash[name] = [files[j]];
			}else{	// Add at right point for date
				for( var k = 0; k < filesHash[name].length; k++ ){
					if( files[j].epoch > filesHash[name][k].epoch ){	// Insert before
						filesHash[name].splice(k, 0, files[j]);
						break;
					}else if( k==filesHash[name].length-1 ){
						filesHash[name].push(files[j]);
						break;
					}
				}
			}
		}
	}
	
	
	// Now sort filesHash into an array, organise by descending dates
	var rank = [];
	var rankHash = {};
	for( f in filesHash ){
		var id = filesHash[f][0].epoch + filesHash[f][0].name;
		rank.push(id);
		rankHash[id] = filesHash[f];
	}
	rank.sort().reverse();
	
	var files = [];
	for( var i = 0; i < rank.length; i++ ){
		files.push( rankHash[rank[i]] );
	}
	
	
	if( fCallback ){
		fCallback(files);
	}else{
		return files;
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadList getFiles\n" + e.toString());
}
};

ThreadList.prototype.destroy = function(){
	if( !this.gtdinboxPage ) return;
	this.gtdinboxPage = null;
	this._threads.length = 0;
	this._threadsHashId = null;
	this._threadsHashRow = null;
	this._multiToOne.destroy();
};