function ThreadListCustomUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);
	
	this.el = this._doc.createElement("DIV");
		this.el.className = "thread-list-custom";
	
	this._eventObservers.observe( this.el, "mousedown", this._click, true );
	this._eventObservers.observe( this.el, "contextmenu", GIPROTOTYPE.bind(this, function(event){	
			GIPROTOTYPE.stop(event);
		}), true);
};

ThreadListCustomUI.prototype.load = function(threadList, threads, options){
	// Render a display of all the threads (optionally restrict threadList to threads subset)
	if( !threads ) threads = threadList._threads;
	this.threads = threads;
	this.threadList = threadList;
	options = options || {};
	
	this.el.innerHTML = "";
	
	var elContainer = this._doc.createElement("TABLE");
	for( var i = 0; i < threads.length; i++ ){
		
		var elRow = this._doc.createElement("TR");
			elRow.className = "row";
			elRow.setAttribute("thread-id", threads[i].id);
			if( (i%2)!=0 ) elRow.className += " alternate";
		
		var tooltip = "";
		
		if( !options.excludeNames ){
			var elNames = this._doc.createElement("TD");
				elNames.className = "names";
				elNames.innerHTML = threads[i].rowNamesHTML;
			elRow.appendChild( elNames );
		}
		if( options.tooltipNames ) tooltip += "Authors: " + threads[i].rowNamesHTML.replace(/<[^>]+>/g,"") + ", ";
		
		if( !options.excludeDetails ){
			var elDetails = this._doc.createElement("TD");
				elDetails.className = "details";
				var html = [];
				if( !options.excludeLabels ){
					for( var j = 0; j < threads[i].labels.length; j++ ){
						html.push("<span class='label'>" + threads[i].labels[j] + "</span>");
					}
				}
				if( !options.excludeSubject ){
					html.push("<span class='subject'>" + threads[i].subject + "</span>");
				}
				elDetails.innerHTML = html.join("");
			elRow.appendChild( elDetails );
		}
		if( options.tooltipSubject ) tooltip += "Subject: " + threads[i].subject + " ";
		if( options.tooltipLabels ) tooltip += "[" + threads[i].labels.join(", ") + "]";
		if( options.tooltipSubject || options.tooltipLabels ) tooltip += ", ";
		
		if( !options.excludeDate ){
			var elDate = this._doc.createElement("TD");
				elDate.className = "date";
				elDate.innerHTML = threads[i].rowDateHTML;
			elRow.appendChild( elDate );
		}
		if( options.tooltipSubject ) tooltip += "Date: " + threads[i].rowDateHTML.replace(/<[^>]+>/g, "") + ", ";
		
		if( !options.excludeCmds ){
			var elPreview = this._doc.createElement("TD");
				elPreview.className = "preview";
				elPreview.innerHTML = "<img src='chrome://gtdinbox/content/gtd_lineitem.png' />";
			elRow.appendChild( elPreview );
		}
		
		
		if( tooltip ){
			tooltip = tooltip.replace(/, $/, "");
			elRow.title = tooltip;
		}
		elContainer.appendChild(elRow);
		
	}
	this.el.appendChild( elContainer );
};

ThreadListCustomUI.prototype._click = function(event){

try{
	var el = GIPROTOTYPE.element(event);
	
	
	var elRow = el;
	while( elRow.parentNode && !elRow.getAttribute("thread-id") ){
		elRow = elRow.parentNode;
	}
	if( !elRow.parentNode ) return;
	var thread = this.threadList.getThreadById(elRow.getAttribute("thread-id"));
	if( !thread ) return;
		
	GIPROTOTYPE.stop(event);
	if( event.button==2 ){	// Right clicked - preview
		this._showPreview( thread, event );
	}else if(el.nodeName=="IMG" && el.parentNode.className=="preview" ){	// Clicked Preview
		this._showPreview( thread, null, el );
	}else{	// Regular click
		var url = this._gtdinboxPage.gmailDOM.local("domain") + "#inbox/" + thread.id;
		if( event.button==0 ){
			this._doc.defaultView.top.location.href = url;
		}else if(event.button==1){
			gBrowser.selectedTab = gBrowser.addTab(url);
		}
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadListCustomUI _click\n" + e.toString());
}
};

ThreadListCustomUI.prototype._showPreview = function( thread, event, el ){
	// Load (create if required) a new popup for this thread. Show at either event or el.
try{	
	if( !this.elPopup ){	// Create popup
		this.elPopup = this._doc.createElement("DIV");
			this.elPopup.className = "thread-list-custom-popup";
			this.elPopup.style.display = "none";
		var elHeader = this._doc.createElement("DIV");
			elHeader.className = "header";
		var elClose = this._doc.createElement("IMG");
			elClose.src = "chrome://gtdinbox/content/cross.png";
			elClose.className = "close-button";
		elHeader.appendChild( elClose );
		this.elPopup.appendChild( elHeader );
		this.elPreview = this._doc.createElement("DIV");
			this.elPreview.className = "preview-box";
		this.elPopup.appendChild( this.elPreview );
		
		this._doc.body.appendChild( this.elPopup );
		
		this.popuper = new GtdPopuper(this._gtdinboxPage, this.elPopup, this);
		
		this._eventObservers.observe( elClose, "click", GIPROTOTYPE.bind(this, function(){	// Close popup
				this.popupOff()
			}), false);
	}
	
		
	// Show it
	this.elPreview.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif' class='loading' />";
	if( event ){
		this.popup(event, {location: "bottomLeft", margin: 15});
	}else{
		this.popup(el);
	}
	
	var fRenderPreview = GIPROTOTYPE.bind(this, function(){	// Render preview body
			this.elPreview.innerHTML = thread.getLatestMsg();
		});
	if( thread.getLatestMsg()=="" ){	// No message, reload 
		this._gtdinboxPage.gmailDOM.getThreadData( [thread.id], this.threadList, fRenderPreview );
	}else{	// Show now
		fRenderPreview();
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadListCustomUI _showPreview\n" + e.toString());
}

};

ThreadListCustomUI.prototype.destroy = function(){
	this._eventObservers.destroy();
	this._gtdinboxPage = null;
	this._doc = null;
	if( this.popuper ) this.popuper.destroy();
	if( this.elPopup ){
		if( this.elPopup.parentNode ) this.elPopup.parentNode.removeChild( this.elPopup );
		this.elPopup = null;
	}
	if( this.el.parentNode ) this.el.parentNode.removeChild( this.el );
	this.el = null;
	this.threads = null;
	this.threadList = null;
}