/* Copyright Andy Mitchell 2007. No warranties */

function ThreadListPopupUI(gtdinboxPage, elParent, inTL){
	this.gtdinboxPage = gtdinboxPage;
	this.doc = gtdinboxPage.doc;
	this.settings = gtdinboxPage.settings;
	this._labels = gtdinboxPage._labels;
	this.threadList = gtdinboxPage._threadList;
	this.threadListUI = gtdinboxPage._threadListUI;
		
	this.eventObservers = new GtdEventObservers(this);
		
	this.create(elParent);
	this.popuper = new GtdPopuper( this.gtdinboxPage, this.el, this );
	this.popuper.addEventHandler( this, "popup", this.onPopup );
	this.popuper.addEventHandler( this, "popupOff", this.onPopupOff );
	this.popuper.addDeadZoneFunction( GIPROTOTYPE.bind(this, this.testPopupDeadZone) );
	
	this._labels.addEventHandler( this, "refresh-labels", this.labelsUpdate );
	
	this.inTL = inTL;
};



ThreadListPopupUI.prototype.create = function(elParent){

try{
	this.el = this.doc.createElement("DIV");
		this.el.className = "thread-list-popup";
		this.el.style.zIndex = 5;
	if( elParent ) elParent.appendChild( this.el );
			
	var elHeader = this.doc.createElement("DIV");
		elHeader.className = "header";
	this.el.appendChild( elHeader );
	this.elTitle = this.doc.createElement("SPAN");
		this.elTitle.className = "title";
		this.elTitle.innerHTML = this.gtdinboxPage.stringBundle.getString("ThreadListPopupUI.TitleLoading");
	elHeader.appendChild( this.elTitle );
	var elClose = this.doc.createElement("IMG");
		elClose.src = "chrome://gtdinbox/content/cross.png";
		elClose.className = "close-button";
		this.eventObservers.observe(elClose, "mousedown", this.clickClose, false);
	elHeader.appendChild(elClose);
	
	this.elLoading = this.doc.createElement("DIV");
		this.elLoading.className = "loading";
		this.elLoading.style.display = "none";
	this.el.appendChild( this.elLoading );
	this.elLoading.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif'/>";
	
	this.elContent = this.doc.createElement("DIV");
	this.el.appendChild( this.elContent );
	
	this.threadActions = new ThreadActionsUI(this.gtdinboxPage);
	elHeader.appendChild( this.threadActions.el );
	this.threadActions.addEventHandler(this, "click-finished", this.clickClose);
	this.labelSelect = new LabelSelectUI(this.gtdinboxPage, this.elContent, null, this.threadActions);
	this.labelSelect.addEventHandler(this, "click-label", this.toggleLabel);
		
	this.threadPreview = new ThreadPreviewUI(this.gtdinboxPage);
	this.previewShowhider = new Showhider( this.gtdinboxPage, "threadlistpopup-preview", {doc: this.doc, elParent: this.elContent, elContent: this.threadPreview.el, title: this.gtdinboxPage.stringBundle.getString("ThreadListPopupUI.PreviewMessage"), show: true});
	GIPROTOTYPE.addClassName( this.previewShowhider.el, "previewer" );

}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadListPopupUI create\n" + e.toString());
}
};



/* LOADING */

ThreadListPopupUI.prototype.loading = function(v){
try{
	if( typeof v=="undefined" ) return this.elContent.style.display=="none";
	this.elContent.style.display = v? "none" : "";
	this.elLoading.style.display = v? "" : "none";
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadListPopupUI loading\n" + e.toString());
}
};

ThreadListPopupUI.prototype.load = function(thread){
	this.loading(true);
	this.thread = thread;	
	this.thread.initEventHandler();
	this.thread.eventHandlers.reset();	// get it exclusively for us
		
	//this.gtdinboxPage._recentItems.addThread(this.thread);
	
	this._labels.getActiveLabels(this, function(activeLabels){
			//this._setRow_refreshUI_ActiveLabelsCallback(activeLabels, row);
			
			var labels = this.thread.verifyLabels( activeLabels );	// ensure it's up to date
			
			
			/*
			if( row ){
				while( row.executeAfterNextPopupRefresh.length > 0 ){
					row.executeAfterNextPopupRefresh[0]();
					row.executeAfterNextPopupRefresh.splice(0,1);
				}
			}
			*/
			this.labelSelect.refresh(this.thread);
			this.threadActions.refresh(this.thread);
			this.threadPreview.load(this.thread);
			
			this.loading(false);
		});
};

ThreadListPopupUI.prototype.loadByRow = function(row){
	this.loading(true);
	this.elTitle.innerHTML = row.subject;
	
	
	row.thread( GIPROTOTYPE.bind(this, function(thread){
		this.thread = thread;
		if( this.thread ){
			this.load(this.thread);
		}else{
			this.retryRowThread(true, row);
		}
	}) );

};
ThreadListPopupUI.prototype.retryRowThread = function(v, row){
	if( this._retryInterval ){
		clearInterval( this._retryInterval );
		this._retryInterval = true;
	}
	if( v ){
		var fRetry = GIPROTOTYPE.bind(this, function(){
				row.thread(GIPROTOTYPE.bind(this, function(thread){
					if( thread ){
						this.retryRowThread(false);
						this.load(thread);
					}
				}));
			});
		this._retryInterval = setInterval( fRetry, 750 );
	}
};




/* POPUP EVENTS */
ThreadListPopupUI.prototype.onPopup = function(){
	if( this.inTL ) this._checked = this.threadListUI.checkRows(null, false);
	
	
	this.doc.body.firstChild.style.opacity = "0.7";
	
	
};
ThreadListPopupUI.prototype.onPopupOff = function(){
	if( this.inTL && this._checked ){
		this.threadListUI.checkRows( this._checked );
		this._checked = null;
	}
	
	this.doc.body.firstChild.style.opacity = "1";
};
ThreadListPopupUI.prototype.testPopupDeadZone = function(el){
	// Return true if a dead zone, thus stopping the popup from closing
	
	// Ensure it's not the New Label Ok/Cancel that is clicked:
	var elNewLabel = this.gtdinboxPage.gmailDOM.element("new-label-box");
	if( elNewLabel ){
		while( el.parentNode ){
			if( el==elNewLabel ) return true;
			el = el.parentNode;
		}
	}
};

ThreadListPopupUI.prototype.clickClose = function(event){
	this.retryRowThread(false);
	this.popupOff();
};

/* EXTERNAL UPDATE EVENTS */
/*
ThreadListPopupUI.prototype.threadUpdate = function(){
	// TODO Only if visible
	if( !this.poppedUp() ) return;
	this.labelSelect.refresh(this.thread);
	this.threadActions.refresh(this.thread);
	this.threadPreview.load(this.thread);
};
*/

ThreadListPopupUI.prototype.labelsUpdate = function(){
	// TODO Only if visible
try{
	if( !this.poppedUp() ) return;
	//this.threadListUI.uncheckOutOfView();	// causes problems on sent items
	this._labels.getActiveLabels(this, function(activeLabels){
			this.thread.verifyLabels( activeLabels );
			this.labelSelect.refresh(this.thread);
			this.threadActions.refresh(this.thread);
			this.threadPreview.load(this.thread);
		});
}catch(e){
	if( GTDDEBUGMODE ) alert("threadlistpopupui labelsUpdate\n" + e.toString());
}
};

ThreadListPopupUI.prototype.toggleLabel = function(e, label){
	// Inform Gmail of change
		
try{
	
	//var on = this.thread? !this.thread.hasLabel(label.labelname) : undefined;
	var labelname = label.labelname;
	var fCallback = GIPROTOTYPE.bind(this, function(act){
			this.labelSelect.selected( this.thread.labels );
			if( act==12 ){	// it's just added it
				if( label.doubleClick || (labelname==(this.gtdinboxPage.settings.labels.types.Statuses.prefix + this.gtdinboxPage.settings.labels.types.Statuses.defaults.Finished)) ){	// clicked finish, archive it
					var threadActionsUI = new ThreadActionsUI(this.gtdinboxPage, null, true);
					threadActionsUI._clickArchive(null, GIPROTOTYPE.bind(this, function(){this.clickClose()}) );
				}
			}
		});
	this.gtdinboxPage._labels.labelThread({labelname:label.labelname, 'on':undefined, 'threads':this.thread, 'fCallback':fCallback});
	
}catch(e){
	if( GTDDEBUGMODE ) alert("threadlistpopupui clicklabel\n" + e.toString());
}
};




/*
ThreadListPopupUI.prototype.setRow = function(row){
	// Update header, refresh labels (+ set selected), update preview
	
	try{
		this.elTitle.innerHTML = row.subject;
			
		row.thread( GIPROTOTYPE.bind(this, function(thread){
			this.thread = thread;
			if( this.thread ){
				this._setRow_refreshUI(this.thread, row);
			}else{
				this.retryRowThread(true,row);
			}
		}) );

	}catch(e){
		if( GTDDEBUGMODE ) alert("popupui setRow\n" + e.toString());
	}
};
ThreadListPopupUI.prototype._setRow_refreshUI = function(thread, row){
try{
	if( !thread ) return;
	this.thread = thread;
	
	
	this.thread.initEventHandler();
	this.thread.eventHandlers.reset();	// get it exclusively for us
	
	
	this.gtdinboxPage._recentItems.addThread(this.thread);
	
	//this.threadListUI.uncheckOutOfView();	// causes problems on sent items
	this._labels.getActiveLabels(this, function(activeLabels){
			this._setRow_refreshUI_ActiveLabelsCallback(activeLabels, row);
		});
	//this._labels.getActiveLabels(null, GIPROTOTYPE.bind(this, function(activeLabels){this._setRow_refreshUI_ActiveLabelsCallback(activeLabels)}) );
	
}catch(e){
	if( GTDDEBUGMODE ) alert("popupui _setRow_refreshUI\n" + e.toString());
}
};
ThreadListPopupUI.prototype._setRow_refreshUI_ActiveLabelsCallback = function(activeLabels, row){

	var labels = this.thread.verifyLabels( activeLabels );	// ensure it's up to date
			
	this.labelSelect.refresh(this.thread);
	//this.labelSelect.newLabels( this.thread.getUnknownBodyLabels() );
	if( row ){
		while( row.executeAfterNextPopupRefresh.length > 0 ){
			row.executeAfterNextPopupRefresh[0]();
			row.executeAfterNextPopupRefresh.splice(0,1);
		}
	}
	this.threadActions.refresh(this.thread);
		
	this.elPreview.innerHTML = this.thread.getLatestMsg();
	
};
*/






