function TabBox(doc, id, options){
	this.id = id;
	this._doc = doc;
	this._eventObservers = new GtdEventObservers(this);
	this._eventHandlers = new GtdEventHandlers_Mutator(this);
	options = options || {};
	this._gtdinboxPage = options.gtdinboxPage;
	
	if( options.el ){
		this.el = options.el;
	}else{
		this.el = this._doc.createElement("DIV");
		if( options.elParent ) options.elParent.appendChild( this.el );
	}
	GIPROTOTYPE.addClassName( this.el, "tab-box" );
	
	if( options.elHeader ){
		this.elHeader = options.elHeader;
	}else{
		this.elHeader = this._doc.createElement("DIV");
		this.el.appendChild( this.elHeader );
	}
	GIPROTOTYPE.addClassName( this.elHeader, "header" );
	this.elClear = this._doc.createElement("DIV");
		this.elClear.className = "clear";
	this.elHeader.appendChild( this.elClear );
	
	if( options.elContent ){
		this.elContent = options.elContent;
	}else{
		this.elContent = this._doc.createElement("DIV");
		this.el.appendChild( this.elContent );
	}
	GIPROTOTYPE.addClassName( this.elContent, "content" );
	
	this._eventObservers.observe( this.elHeader, "mousedown", this._clickHeader, true );
	
	
};

TabBox.prototype.loading = function(v){
	if( typeof v=="undefined" ) return !!(this._loading);
	
	if( v ){
		if( !this._loading ){
			this._loading = this._doc.createElement("IMG");
				this._loading.className = "loading";
				this._loading.src = "chrome://gtdinbox/skin/indicator_medium.gif";
			for( var i = 0; i < this.el.childNodes.length; i++ ) this.el.childNodes[i].style.display = "none";
			this.el.appendChild( this._loading );
		}
	}else{
		if( this._loading ){
			this._loading.parentNode.removeChild( this._loading );
			this._loading = null;
			for( var i = 0; i < this.el.childNodes.length; i++ ) this.el.childNodes[i].style.display = "";
		}
	}
};

TabBox.prototype.add = function( name, title, elContent ){

try{
	if( this.title(name)==null ){	// does not exist
		var elTitle = this._doc.createElement("DIV");
			elTitle.setAttribute("tabboxname", name);
			elTitle.innerHTML = title;
		var elContentContainer = this._doc.createElement("DIV");
			elContentContainer.setAttribute("tabboxname", name);
			if( elContent ) elContentContainer.appendChild( elContent );
		this.elHeader.insertBefore( elTitle, this.elClear );
		this.elContent.appendChild( elContentContainer );
		
		if( this.selected() ){	// Refresh selection
			this.selected(this.selected());
		}else{	// Nothing is selected
			this.selected(name);
		}
	}	
	return {'name':name, 'title':this.title(name), 'headerEl': this._getHeaderEl(name), 'contentEl': this.contentEl(name)};
	
}catch(e){
	if( GTDDEBUGMODE ) alert("TabBox add\n" + e.toString());
}
};

TabBox.prototype.selected = function(name){
try{
	// Get the name or remove selected from header
	var newSelection = false;
	for( var i = 0; i < this.elHeader.childNodes.length - 1; i++ ){
		if( name==this.elHeader.childNodes[i].getAttribute("tabboxname") ){	// Select it
			if( !GIPROTOTYPE.hasClassName( this.elHeader.childNodes[i], "selected" ) ) GIPROTOTYPE.addClassName(this.elHeader.childNodes[i], "selected" );
			newSelection = true;
		}else{	// Try to remove selected
			if( GIPROTOTYPE.hasClassName( this.elHeader.childNodes[i], "selected" ) ){
				if( typeof name=="undefined" ){	// Return name
					return this.elHeader.childNodes[i].getAttribute("tabboxname");
				}else{	// Remove selected
					GIPROTOTYPE.removeClassName( this.elHeader.childNodes[i], "selected" );
				}
			}
		}
	}
	
	
	if( typeof name!="undefined" ){
		// Remove selected from content and add to the new one
		for( var i = 0; i < this.elContent.childNodes.length; i++ ){
			if( name==this.elContent.childNodes[i].getAttribute("tabboxname") ){	// Select it
				if( !GIPROTOTYPE.hasClassName( this.elContent.childNodes[i], "selected" ) ) GIPROTOTYPE.addClassName(this.elContent.childNodes[i], "selected" );
				this.elContent.childNodes[i].style.display = "";
			}else{	// Try to remove it
				if( GIPROTOTYPE.hasClassName( this.elContent.childNodes[i], "selected" ) ){
					GIPROTOTYPE.removeClassName( this.elContent.childNodes[i], "selected" );
				}
				this.elContent.childNodes[i].style.display = "none";
			}
		}
	}
	
	if( newSelection ) this._eventHandlers.broadcast("selected");
	return "";
	
}catch(e){
	if( GTDDEBUGMODE ) alert("TabBox selected\n" + e.toString());
}
};

TabBox.prototype._getHeaderEl = function(name){
try{
	for( var i = 0; i < this.elHeader.childNodes.length - 1; i++ ){
		if( this.elHeader.childNodes[i].getAttribute("tabboxname")==name ) return this.elHeader.childNodes[i];
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("TabBox _getHeaderEl\n" + e.toString());
}
};


TabBox.prototype.title = function(name, title){
try{
	for( var i = 0; i < this.elHeader.childNodes.length - 1; i++ ){
		if( this.elHeader.childNodes[i].getAttribute("tabboxname")==name ){
			if( typeof title=="undefined" ){
				return this.elHeader.childNodes[i].innerHTML || "";
			}else{
				this.elHeader.childNodes[i].innerHTML = title;
			}
		}
	}
	return null;
}catch(e){
	if( GTDDEBUGMODE ) alert("TabBox title\n" + e.toString());
}
};

TabBox.prototype.enabled = function(name, v){
	if( typeof v=="undefined" ){
		return !GIPROTOTYPE.hasClassName(this._getHeaderEl(name), "disabled");
	}else{
		if( v ){
			GIPROTOTYPE.removeClassName( this._getHeaderEl(name), "disabled" );
		}else{
			GIPROTOTYPE.addClassName( this._getHeaderEl(name), "disabled" );
			this._selectFirstAvailable();
		}
	}
};

TabBox.prototype._selectFirstAvailable = function(){
	if( this.selected()==name ){	// Change selection
		for( var i = 0; i < this.elHeader.childNodes.length - 1; i++ ){
			if( !GIPROTOTYPE.hasClassName( this.elHeader.childNodes[i], "disabled" ) && this.elHeader.childNodes[i].style.display!="none" ) this.selected( this.elHeader.childNodes[i].getAttribute("tabboxname") );
		}
	}
};

TabBox.prototype.visible = function(name, v){
	if( typeof v=="undefined" ){
		return (this._getHeaderEl(name).style.display!=none);
	}else{
		this._getHeaderEl(name).style.display = v? "" : "none";
		this.contentEl(name).style.display = v? "" : "none";
		if( !v ) this._selectFirstAvailable();
	}
};

TabBox.prototype.numberOfActiveTabs = function(){
	// Get a count of number of inuse tabs
	var c = 0;
	for( var i = 0; i < this.elHeader.childNodes.length - 1; i++ ){
		var el = this.elHeader.childNodes[i];
		if( !GIPROTOTYPE.hasClassName(el, "disabled") && el.style.display!="none" ) c++;
	}
	return c;
};

TabBox.prototype.contentEl = function(name, elContent){
try{
	for( var i = 0; i < this.elContent.childNodes.length; i++ ){
		if( this.elContent.childNodes[i].getAttribute("tabboxname")==name ){
			if( typeof elContent=="undefined" ){
				return this.elContent.childNodes[i];
			}else{
				this.elContent.childNodes[i].innerHTML = "";
				this.elContent.childNodes[i].appendChild( elContent );
			}
		}
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("TabBox contentEl\n" + e.toString());
}
};



TabBox.prototype._clickHeader = function(event){
	// Get the name, make sure it's selected, broadcast
try{
	var el = GIPROTOTYPE.element(event);
	while( el.parentNode && !el.getAttribute("tabboxname") ){
		el = el.parentNode;
	}
	if( !el.getAttribute("tabboxname") ) return;
	if( GIPROTOTYPE.hasClassName( el, "disabled" ) ) return;
	
	var name = el.getAttribute("tabboxname");
	
	this.selected(name);
	
	this._eventHandlers.broadcast("selected-by-header-click");
}catch(e){
	if( GTDDEBUGMODE ) alert("TabBox _clickHeader\n" + e.toString());
}
};

